@@ -141,6 +141,20 @@ class DepsGraph(object):
         open_nodes = nodes_by_level[1]
         return open_nodes
 
+    def full_closure(self, node):
+        closure = OrderedDict()
+        current = node.neighbors()
+        while current:
+            new_current = []
+            for n in current:
+                closure[n] = n
+            for n in current:
+                for neigh in n.public_neighbors():
+                    if neigh not in new_current and neigh not in closure:
+                        new_current.append(neigh)
+            current = new_current
+        return closure
+
     def closure(self, node):
         closure = OrderedDict()
         current = node.neighbors()
