@@ -246,6 +246,29 @@ class ConanManager(object):
 
         return deps_graph, graph_updates_info, self._get_project_reference(reference, conanfile)
 
+    def install_workspace(self, profile, workspace, remote_name, build_modes, update):
+        loader = self.get_loader(profile)
+        references = [ConanFileReference(v, "root", "project", "develop") for v in workspace.root]
+        conanfile = loader.load_virtual(references)
+        remote_proxy = self.get_proxy(remote_name=remote_name)
+        graph_builder = self._get_graph_builder(loader, remote_proxy)
+        graph_builder._workspace = workspace
+        deps_graph = graph_builder.load_graph(conanfile, False, update)
+
+        output = ScopedOutput(str("Workspace"), self._user_io.out)
+        output.highlight("Installing...")
+        print_graph(deps_graph, self._user_io.out)
+
+        build_mode = BuildMode(build_modes, self._user_io.out)
+        build_requires = BuildRequires(loader, graph_builder, self._registry)
+        installer = ConanInstaller(self._client_cache, output, remote_proxy, build_mode,
+                                   build_requires, recorder=self._recorder)
+        installer._workspace = workspace
+
+        installer.install(deps_graph, profile.build_requires, keep_build=False, update=update)
+        build_mode.report_matches()
+        workspace.generate()
+
     def install(self, reference, install_folder, profile, remote_name=None, build_modes=None,
                 update=False, manifest_folder=None, manifest_verify=False,
                 manifest_interactive=False, generators=None, no_imports=False, inject_require=None,
@@ -282,6 +305,7 @@ class ConanManager(object):
         conanfile = self._load_install_conanfile(loader, reference)
         if inject_require:
             self._inject_require(conanfile, inject_require)
+
         graph_builder = self._get_graph_builder(loader, remote_proxy)
         deps_graph = graph_builder.load_graph(conanfile, False, update)
 
